local les = {e = {}, s = {},} -- all public functions

local templates  = {} -- for entity component complience TODO
local systems		 = {} -- all systems by uid
local entities   = {} -- all entities by uid
local sets	     = {} -- list of _meta systems from queries
local categories = {} -- lists of systems by category

function les.component(name, data)
	templates[name] = data
end

function les.e.set(components)
	local index = #entities+1
	local entity = {}
	for name,data in pairs(components) do
		entity[name] = data
	end
	entity._meta = {index = index}
	entities[index] = entity
	-- register the added entity with matching systems
	for i,system in pairs(systems) do
		if system.reqs(entity) then
			system._set[index] = entity
		end
	end
  return index, entity
end

function les.s.set(of, sort, reqs, func)
	local system = {
	  [of] = true,
		sort = sort,
		reqs = reqs,
		func = func,
		_set = {},
	}
	if not categories[of] then categories[of] = {} end
	table.insert(categories[of], system)
	table.sort(categories[of], function(a, b)
		return a.sort < b.sort and true or false
	end)
	-- register matching entities with the added system
	for i,entity in pairs(entities) do
		if system.reqs(entity) then
			system._set[entity._meta.index] = entity
		end
	end
  systems[#systems+1] = system
  return system
end

function les.run(of, ...)
	if not categories[of] then return end
	for i,system in ipairs(categories[of]) do
		for j,entity in pairs(system._set) do
			system.func(entity, ...)
		end
	end
end

function les.e.all(uid, reqs)
	if not sets[uid] then
		sets[uid] = les.s.set("_meta", 1, reqs, nil)
	end
	return sets[uid]._set
end

function les.e.get(index)
	return entities[index]
end

function les.e.del(id)
	local index
	if type(id) == "number" then index = id
	else index = id._meta.index end

	for i,system in pairs(systems) do
		system._set[index] = nil
	end
	entities[index] = nil
end

return les
