local state

function love.load(arg)
  if arg[#arg] == "-debug" then require("mobdebug").start() end
 
	les = require "les"

	local w = love.window.getWidth()
	local h = love.window.getHeight()

	local c = 10

	local f = love.graphics.getFont()

	--[[
	les.s.set("draw", 2,
		function(e) return e.dbg and e.pos end,
		function(e, ...)
			love.graphics.print("x: "..e.pos.x.."\ny: "..e.pos.y, 10, 10)
		end)
		]]

	les.s.set("draw", 2,
		function(e) return e.score end,
		function(e, ...)
			love.graphics.print(e.score.count, w-10-f:getWidth(e.score.count), 10)
				local color = e.score.time*255/e.score.tts
				love.graphics.setColor(255, color, color)
			love.graphics.print(string.format("%01.1f", e.score.time), 10, 10)
		end)

	les.s.set("key", 1,
		function(e) return e.segments end,
		function(e, ...)
			local key = select(1, ...)
			local s = les.e.get(e.segments[1])
			if s.go.y == 0 then
				if key == "down" then s.go.y = 1
				elseif key == "up" then s.go.y = -1 end
				if s.go.y ~= 0 then s.go.x = 0 end
			end
			if s.go.x == 0 then
				if key == "right" then s.go.x = 1
				elseif key == "left" then s.go.x = -1 end
				if s.go.x ~= 0 then s.go.y = 0 end
			end
		end)

	les.s.set("step", 1,
		function(e) return e.segments end,
		function(e, ...)
			local goA
			local goB
			for i,id in ipairs(e.segments) do
				local segment = les.e.get(id)
				segment.pos.x = segment.pos.x + segment.go.x * e.step
				segment.pos.y = segment.pos.y + segment.go.y * e.step
				if segment.pos.x < 0 then
					segment.pos.x = w-c
				elseif segment.pos.x+c > w then
					segment.pos.x = 0
				end
				if segment.pos.y < 0 then
					segment.pos.y = h-c
				elseif segment.pos.y+c > h then
					segment.pos.y = 0
				end
				goA = {x = segment.go.x, y = segment.go.y}
				if goB then
					segment.go.x = goB.x
					segment.go.y = goB.y
				end
				goB = {}
				goB.x = goA.x
				goB.y = goA.y
			end
		end)

	les.s.set("step", 2,
		function(e) return e.segments end,
		function(e, ...)
			local s = les.e.get(e.segments[1])
			for i,id in ipairs(e.segments) do
				local segment = les.e.get(id)
				if i ~= 1 and
					s.pos.x == segment.pos.x and
					s.pos.y == segment.pos.y then
					s.color.g = 0
					les.e.get(state).dead = true
				end
			end
		end)

	les.s.set("update", 3,
		function(e) return e.score end,
		function(e, ...)
			local dt = select(1, ...)
			e.score.time = e.score.time - dt
			if e.score.time < 0 then e.score.time = 0 end
		end)

	les.s.set("update", 1,
		function(e) return e.timer and e.segments and e.step end,
		function(e, ...)
			local dt = select(1, ...)
			e.timer.elapsed = e.timer.elapsed + dt
			if e.timer.elapsed >= e.timer.lap then
				les.run("step")
				e.timer.elapsed = e.timer.elapsed - e.timer.lap
			end
		end)
	
	les.s.set("draw", 1,
		function(e)
			return e.draw and
				e.draw.of == "rectangle" and
				e.draw.layer == 1
		end,
		function(e, ...)
			love.graphics.setColor(e.color.r, e.color.g, e.color.b)
			local d = (c - e.size) / 2
			love.graphics[e.draw.of]("fill",
				e.pos.x+d,  e.pos.y+d, e.size, e.size)
		end)

	les.s.set("draw", 2,
		function(e)
			return e.draw and
				e.draw.of == "rectangle" and
				e.draw.layer == 2
		end,
		function(e, ...)
			love.graphics.setColor(e.color.r, e.color.g, e.color.b)
			local d = (c - e.size) / 2
			love.graphics[e.draw.of]("fill",
				e.pos.x+d,  e.pos.y+d, e.size, e.size)
		end)

	les.s.set("update", 2,
		function(e) return e.segments and e.food end,
		function(e, ...)
			local food = les.e.get(e.food)
			local head = les.e.get(e.segments[1])
			if math.floor(food.pos.x / c) == math.floor(head.pos.x /c) and
				math.floor(food.pos.y / c) == math.floor(head.pos.y /c) then
				local score = les.e.get(state).score
				if score.time > 0 then
					score.count = score.count + 1
				end
				score.time = score.tts
				repeat
					food.pos.x = math.random(1, w/c) * c - c
					food.pos.y = math.random(1, h/c) * c - c
				until (function()
					for i,id in ipairs(e.segments) do
						local segment = les.e.get(id)
						if math.floor(food.pos.x / c) == math.floor(segment.pos.x /c) and
							math.floor(food.pos.y / c) == math.floor(segment.pos.y /c) then
							return false
						end
					end
					return true
				end)()
				local foot = les.e.get(e.segments[#e.segments])
				local new = les.e.set({
					color = {r = 255, g = 255, b = 255},
					draw = {of = "rectangle", layer = 1},
					pos = {x = foot.pos.x, y = foot.pos.y},
					go = {x = 0, y = 0},
					size = c - c / 10,
				})
				table.insert(e.segments, new)
			end
		end)

	local food = les.e.set({
		color = {r = 255, g = 0, b = 255},
		draw = {of = "rectangle", layer = 1},
		pos = {x = 100, y = 100},
		size = c - c / 5,
	})

	local head = les.e.set({
		color = {r = 255, g = 255, b = 0},
		draw = {of = "rectangle", layer = 2},
		pos = {x = 0, y = 0},
		go = {x = 1, y = 0},
		size = c - c / 10,
		dbg = true,
	})

	les.e.set({
		segments = {head},
		timer = {elapsed = 0.1, lap = 0.1},
		step = c,
		food = food,
	})

	state = les.e.set({
		dead = false,
		score = {count = 0, tts = 3, time = 3},
	})
end

function love.keyreleased(key)
	les.run("key", key)
end

function love.update(dt)
	if les.e.get(state).dead then
		return
	end
	les.run("update", dt)
end

function love.draw()
	les.run("draw")
end

function round(num, idp)
	local mult = 10^(idp or 0)
	return math.floor(num * mult + 0.5) / mult
end
